/*
	$Id: DalInformixDatabase.h 1198 2018-11-21 13:17:38Z john $

	This file is part of the RFC DAL.

	The RFC DAL is Copyright (c) 2004 - 2018, The 4c Company.

	This component is PROPRIETARY software and the exclusive property of
	The 4c Company. In order to use it in conjunction with RFC DAL, you
	must use the LGPL distribution of that software. You can use this
	component and modify it exclusively under the terms of The 4c
	Company proprietary software license, available at
	http://www.4ccompany.com

	The RFC DAL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef __DalInformixDatabase_h__
#define __DalInformixDatabase_h__

#include <librfcdal/DalDatabase.h>
#include <librfcdal/DalDatabaseException.h>

#include <string>

#if defined (RFCDALDB_EXPORT)
#	undef RFCDALDB_EXPORT
#endif /* RFCDALDB_EXPORT */

#if defined (LINUX)
#	define RFCDALDB_EXPORT
#elif defined (_WINDOWS)
#	include <windows.h>
#	if defined (LIBRFCDALDBINFORMIX_EXPORTS)
#		define RFCDALDB_EXPORT __declspec( dllexport )
#	else
#		define RFCDALDB_EXPORT __declspec( dllimport )
#	endif /* LIBRFCDALDBINFORMIX_EXPORTS */
#endif /* LINUX */

typedef void* SQLHANDLE;
typedef signed short int SQLSMALLINT;
typedef SQLSMALLINT SQLRETURN;

namespace RfcDal
{
	class RFCDALDB_EXPORT DalInformixDatabase : public DalDatabase
	{
		private:
			static SQLHANDLE _hEnv;
			SQLHANDLE _hConn;
			static DalDatabase::ConnectionParameterList _parameterS;

		public:
			DalInformixDatabase();
			virtual ~DalInformixDatabase();

			bool connect( const std::string& strDSN,
							const std::string& strUserId, const std::string& strPassword );
			virtual const ConnectionParameterList& connectionParameters() const;
			virtual bool connectX( const std::string& strConnect, ConnectionParameterMap& map = _mapDefault );
			virtual bool connectX( ConnectionParameterMap& map );
			virtual void disconnect();

			virtual unsigned execute( const SqlModifyStatement& statement );
			virtual unsigned execute( Result& result, const SqlQueryStatement& statement, SqlQueryInfo* pinfo = NULL );

			virtual bool autoCommit();
			virtual void setAutoCommit( bool bOn );
			virtual void begin();
			virtual void commit();
			virtual void rollback();

			virtual void schemas( SqlSchemaList& schemaS, const SqlDatabase& catalog );
			virtual void tables( SqlTableList& tableS, const SqlSchema& schema );
			virtual void invoke( SqlQueryInfo& info, const SqlTable& table );

		protected:
			void translateResult( int nHandleType, SQLHANDLE handle, SQLRETURN rc, int nErrorCode );

		private:
			DalInformixDatabase( const DalInformixDatabase& );
			const DalInformixDatabase& operator=( const DalInformixDatabase& );
	};
};

#endif /* !__DalInformixDatabase_h__ */
